# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import List, Union

from pydantic.v1 import BaseModel, Field, StrictFloat, StrictInt, conlist

from clients.image_gen.models.image_generation import ImageGeneration


class ImageGenerationResponse(BaseModel):
    """
    Response to an ImageGenerationRequest.  # noqa: E501
    """

    images: conlist(ImageGeneration) = Field(
        ..., description="List of ImageGeneration(s) generated by the request."
    )
    prediction_time_ms: Union[StrictFloat, StrictInt] = Field(
        ..., description="Total runtime of the image generation(s)."
    )
    seed: StrictInt = Field(
        ..., description="The seed used for the image generation(s)."
    )
    __properties = ["images", "prediction_time_ms", "seed"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ImageGenerationResponse:
        """Create an instance of ImageGenerationResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in images (list)
        _items = []
        if self.images:
            for _item in self.images:
                if _item:
                    _items.append(_item.to_dict())
            _dict["images"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ImageGenerationResponse:
        """Create an instance of ImageGenerationResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ImageGenerationResponse.parse_obj(obj)

        _obj = ImageGenerationResponse.parse_obj(
            {
                "images": [
                    ImageGeneration.from_dict(_item) for _item in obj.get("images")
                ]
                if obj.get("images") is not None
                else None,
                "prediction_time_ms": obj.get("prediction_time_ms"),
                "seed": obj.get("seed"),
            }
        )
        return _obj
