# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, Dict, Optional

from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr

from clients.asset_lake.models.asset_type import AssetType
from clients.asset_lake.models.transfer_api_type import TransferApiType


class CreateAssetRequest(BaseModel):
    """
    CreateAssetRequest
    """

    asset_type: AssetType = Field(...)
    data: Dict[str, Any] = Field(
        ...,
        description="Data specific to the asset type; use the class named ${asset_type}Data",
    )
    description: Optional[StrictStr] = Field(...)
    is_public: Optional[StrictBool] = False
    name: StrictStr = Field(...)
    transfer_api_type: Optional[TransferApiType] = None
    __properties = [
        "asset_type",
        "data",
        "description",
        "is_public",
        "name",
        "transfer_api_type",
    ]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateAssetRequest:
        """Create an instance of CreateAssetRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict["description"] = None

        # set to None if transfer_api_type (nullable) is None
        # and __fields_set__ contains the field
        if (
            self.transfer_api_type is None
            and "transfer_api_type" in self.__fields_set__
        ):
            _dict["transfer_api_type"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateAssetRequest:
        """Create an instance of CreateAssetRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateAssetRequest.parse_obj(obj)

        _obj = CreateAssetRequest.parse_obj(
            {
                "asset_type": obj.get("asset_type"),
                "data": obj.get("data"),
                "description": obj.get("description"),
                "is_public": obj.get("is_public")
                if obj.get("is_public") is not None
                else False,
                "name": obj.get("name"),
                "transfer_api_type": obj.get("transfer_api_type"),
            }
        )
        return _obj
