from __future__ import annotations

# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from enum import Enum


class BaseEngine(str, Enum):
    """
    BaseEngine
    """

    """
    allowed enum values
    """
    IMAGE_SLASH_STABLE_MINUS_DIFFUSION_MINUS_V1_MINUS_5 = "image/stable-diffusion-v1-5"
    IMAGE_SLASH_STABLE_MINUS_DIFFUSION_MINUS_XL_MINUS_V1_MINUS_0 = (
        "image/stable-diffusion-xl-v1-0"
    )
    TEXT_SLASH_LLAMA_MINUS_2_MINUS_7B = "text/llama-2-7b"

    @classmethod
    def from_json(cls, json_str: str) -> BaseEngine:
        """Create an instance of BaseEngine from a JSON string"""
        return BaseEngine(json.loads(json_str))
