from __future__ import annotations

# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from enum import Enum


class FileExtension(str, Enum):
    """
    FileExtension
    """

    """
    allowed enum values
    """
    UNKNOWN = "unknown"
    PNG = "png"
    JPG = "jpg"
    JPEG = "jpeg"
    TXT = "txt"

    @classmethod
    def from_json(cls, json_str: str) -> FileExtension:
        """Create an instance of FileExtension from a JSON string"""
        return FileExtension(json.loads(json_str))
