from __future__ import annotations

# coding: utf-8

"""
    FastAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from enum import Enum


class SDXLStyles(str, Enum):
    """
    The supported SDXL styles.
    """

    """
    allowed enum values
    """
    BASE = "base"
    ENUM_3D_MINUS_MODEL = "3d-model"
    ANALOG_MINUS_FILM = "analog-film"
    ANIME = "anime"
    CINEMATIC = "cinematic"
    COMIC_MINUS_BOOK = "comic-book"
    CRAFT_CLAY = "Craft Clay"
    MODELING_MINUS_COMPOUND = "modeling-compound"
    DIGITAL_MINUS_ART = "digital-art"
    ENHANCE = "enhance"
    FANTASY_MINUS_ART = "fantasy-art"
    ISOMETRIC = "isometric"
    LINE_MINUS_ART = "line-art"
    LOW_MINUS_POLY = "low-poly"
    NEON_MINUS_PUNK = "neon-punk"
    ORIGAMI = "origami"
    PHOTOGRAPHIC = "photographic"
    PIXEL_MINUS_ART = "pixel-art"
    TILE_MINUS_TEXTURE = "tile-texture"
    ADVERTISING = "Advertising"
    FOOD_PHOTOGRAPHY = "Food Photography"
    REAL_ESTATE = "Real Estate"
    ABSTRACT = "Abstract"
    CUBIST = "Cubist"
    GRAFFITI = "Graffiti"
    HYPERREALISM = "Hyperrealism"
    IMPRESSIONIST = "Impressionist"
    POINTILLISM = "Pointillism"
    POP_ART = "Pop Art"
    PSYCHEDELIC = "Psychedelic"
    RENAISSANCE = "Renaissance"
    STEAMPUNK = "Steampunk"
    SURREALIST = "Surrealist"
    TYPOGRAPHY = "Typography"
    WATERCOLOR = "Watercolor"
    FIGHTING_GAME = "Fighting Game"
    GTA = "GTA"
    SUPER_MARIO = "Super Mario"
    MINECRAFT = "Minecraft"
    POKÉMON = "Pokémon"
    RETRO_ARCADE = "Retro Arcade"
    RETRO_GAME = "Retro Game"
    RPG_FANTASY_GAME = "RPG Fantasy Game"
    STRATEGY_GAME = "Strategy Game"
    STREET_FIGHTER = "Street Fighter"
    LEGEND_OF_ZELDA = "Legend of Zelda"
    ARCHITECTURAL = "Architectural"
    DISCO = "Disco"
    DREAMSCAPE = "Dreamscape"
    DYSTOPIAN = "Dystopian"
    FAIRY_TALE = "Fairy Tale"
    GOTHIC = "Gothic"
    GRUNGE = "Grunge"
    HORROR = "Horror"
    MINIMALIST = "Minimalist"
    MONOCHROME = "Monochrome"
    NAUTICAL = "Nautical"
    SPACE = "Space"
    STAINED_GLASS = "Stained Glass"
    TECHWEAR_FASHION = "Techwear Fashion"
    TRIBAL = "Tribal"
    ZENTANGLE = "Zentangle"
    COLLAGE = "Collage"
    FLAT_PAPERCUT = "Flat Papercut"
    KIRIGAMI = "Kirigami"
    PAPER_MACHE = "Paper Mache"
    PAPER_QUILLING = "Paper Quilling"
    PAPERCUT_COLLAGE = "Papercut Collage"
    PAPERCUT_SHADOW_BOX = "Papercut Shadow Box"
    STACKED_PAPERCUT = "Stacked Papercut"
    THICK_LAYERED_PAPERCUT = "Thick Layered Papercut"
    ALIEN = "Alien"
    FILM_NOIR = "Film Noir"
    HDR = "HDR"
    LONG_EXPOSURE = "Long Exposure"
    NEON_NOIR = "Neon Noir"
    SILHOUETTE = "Silhouette"
    TILT_MINUS_SHIFT = "Tilt-Shift"

    @classmethod
    def from_json(cls, json_str: str) -> SDXLStyles:
        """Create an instance of SDXLStyles from a JSON string"""
        return SDXLStyles(json.loads(json_str))
