# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, Optional

from pydantic.v1 import BaseModel


class NoTransferApi(BaseModel):
    """
    This type exists to avoid Optional[TransferApi] so openapi-generator can work.  In this class, type is not added to TransferApiType because it is not a valid TransferApiType. It exists only to note that no TransferApi is defined.  # noqa: E501
    """

    type: Optional[Any] = None
    __properties = ["type"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NoTransferApi:
        """Create an instance of NoTransferApi from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # set to None if type (nullable) is None
        # and __fields_set__ contains the field
        if self.type is None and "type" in self.__fields_set__:
            _dict["type"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> NoTransferApi:
        """Create an instance of NoTransferApi from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return NoTransferApi.parse_obj(obj)

        _obj = NoTransferApi.parse_obj({"type": obj.get("type")})
        return _obj
