from __future__ import annotations

# coding: utf-8

"""
    asset-lake-api-server

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import json
import pprint
import re  # noqa: F401
from enum import Enum


class DataType(str, Enum):
    """
    DataType
    """

    """
    allowed enum values
    """
    FP16 = "fp16"
    FP32 = "fp32"
    INT4 = "int4"
    INT8 = "int8"

    @classmethod
    def from_json(cls, json_str: str) -> DataType:
        """Create an instance of DataType from a JSON string"""
        return DataType(json.loads(json_str))
